from django import forms
from django.utils.safestring import mark_safe
from django.contrib.admin.widgets import RelatedFieldWidgetWrapper

class AutocompleteWidget(forms.Widget):
	list_display = []
	search_fields = []
	format_data = ''
	service_url = "/complete/"
	model = None

	def __init__(self, model, list_display, search_fields, format_data):
		super(AutocompleteWidget, self).__init__()
		self.model = model
		self.list_display = list_display
		self.search_fields = search_fields
		self.format_data = format_data
    # end def

	def render(self, name, value, attrs={}):
		list_display = ""
		search_fields = ""
		for display in self.list_display:
			list_display = "%s&%s=%s" % (list_display, "list_display", display)
		# end for
		for fields in self.search_fields:
			search_fields = "%s&%s=%s" % (search_fields, "search_fields", fields)
		# end for
		html = """
			<input type="text" id="%(name)s" style="float:left" placeholder="Escribe algo...">
			<input id="id_%(name)s" type="hidden" name="%(name)s" value="%(value)s">
			<script type="text/javascript">
				var url = "%(url)s";
				function load_data(value){
					django.jQuery.ajax({
						'url': "%(url_val)s&query="+value,
						success: function(data){
							if (data.suggestions[0]){
								django.jQuery('#%(name)s').val(data.suggestions[0].value);
							}
						}
					});
				}
				load_data("%(value)s");
				django.jQuery("#id_%(name)s").change(function (data){
					load_data(this.value);
				});
				django.jQuery("#%(name)s").autocomplete({
					serviceUrl: url,
					onSelect: function (suggestion) {
						django.jQuery('[name="%(name)s"]').val(suggestion.date);
					}
				});
			</script>
		""" % {
			'name': name,
			'value': value or '', 
			'url': "%s?module_path=%s&model_name=%s&%s&%s&format_data=%s" % (
				self.service_url,
				self.model.__module__,
				self.model.__name__,
				list_display,
				search_fields, 
				self.format_data
			),
			'url_val': "%s?module_path=%s&model_name=%s&%s&%s&format_data=%s" % (
				self.service_url,
				self.model.__module__,
				self.model.__name__,
				list_display,
				"&search_fields=pk", 
				self.format_data
			)
		}
		return mark_safe(html)
	# end def

	class Media:
		css = {
			'all': ('django_autocomplete/css/jquery.autocomplete.css', )
		}
		js = ('django_autocomplete/js/jquery.autocomplete.js', 'admin/js/admin/RelatedObjectLookups.js')
	# end class
# end class




from django.db import models
import widgets

class AutocompleteRelation(models.ForeignKey):
    def __init__(self, *args, **kwargs):
        super(AutocompleteRelation, self).__init__(*args, **kwargs)
    # end def

# end class


from django.shortcuts import render
from django.http import HttpResponse
from django.db.models import Q
import importlib
import datetime
import json

def complete(request):
	module_path = request.GET.get('module_path', False) 
	model_name = request.GET.get('model_name', False) 
	search_fields = request.GET.getlist('search_fields') 
	list_display = request.GET.getlist('list_display')
	format_data = request.GET.get('format_data', False)
	paginated_by = request.GET.get('paginated_by', '10')
	date_format = "%d/%m/%Y"
	datetime_format = "%d/%m/%Y %I:%M%p"
	query = request.GET.get('query', '') 

	if module_path and model_name and paginated_by.isdigit() and query:
		module = importlib.import_module(module_path)
		if hasattr(module, model_name):
			model = getattr(module, model_name)
			queryset = model.objects.all()
			q = False
			for column in search_fields:
				if query:
					if column != 'pk':
						kwargs = {
							'{0}__{1}'.format(column, 'icontains'): query, 
						}
					else:
						kwargs = {
							'pk': query
						}
					# end if
					if q:
						q = q | Q(**kwargs)
					else:
						q = Q(**kwargs)
					# end if
				# end if
			#end for
			print q
			if q:
				queryset = queryset.filter(q)
			# end if
			rows = []
			for obj in queryset[0:int(paginated_by)]:
				row = {}
				for field in list_display:
					val = getattr(obj, field)
					if isinstance(val, datetime.datetime):
						row[field] = val.strftime(datetime_format)
					elif isinstance(val, datetime.date):
						row[field] = val.strftime(date_format)
					else:
						row[field] = unicode(val)
					# end if
				# end for
				if format_data == '':
					format_data = str(obj)
				# end if

				rows.append({
					'date': str(obj.pk),
					'value': format_data % row
				})
			# end for
			response = {
				"query": query,
				"suggestions": rows
			}
			return HttpResponse(json.dumps(response), content_type="application/json")
		# end if
	# end if
	return HttpResponse("[]")
# end def


from django.contrib import admin

class PublisherStateFilter(admin.FieldListFilter):
    parameter_name = 'status'
    template = 'django_autocomplete/django_autocomplete.html'
    service_url = "/complete/"


    @classmethod
    def make(cls, main_field):
        class Extend(PublisherStateFilter):
            pass
        # end class
        Extend.main_field = main_field
        return Extend
    # end def

    def expected_parameters(self):
        return self.parameter_name
    # end def

    def queryset(self, request, queryset):
    	self.request = request
        pass
    # en ddef

    def choices(self, changelist):
        model = self.field.related_model
        value = self.request.GET.get(self.field_path, '')

        return [{
        	'name': self.field_path,
        	'path': self.request.path,
        	'value': value,
        	'title': self.title.title(),
        	'module': model.__module__,
			'model': model.__name__,
			'url': self.service_url,
			'main_field': self.main_field
        }]
    # end def
# end class


from distutils.core import setup

# version (shape.function.path)
# python setup.py sdist upload
setup(
	name='django-autocomplete-x',
	version='0.0.20',
	packages=['django_autocomplete', ],
	url='https://github.com/exildev/django-autocomplete',
	author="Luis Miguel Morales Pajaro",
	author_email="luismiguel.mopa@gmail.com",
	licence="Creative Common",
	description="django autocomplete plugin",
	platform="Linux",
	zip_safe=False,
	include_package_data=True,
	package_data={'django_autocomplete': [
		'static/django_autocomplete/css/*.css',
		'static/django_autocomplete/js/*.js',
		'templates/django_autocomplete/*.html'
	]},
)